﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace Cjc.Data.Mapping
{
	public class ExpressionVisitor<T> : ExpressionVisitor where T : Expression
	{
		Func<T, Expression> visitor;

		public ExpressionVisitor( Func<T, Expression> visitor )
		{
			this.visitor = visitor;
		}

		public static Expression Visit(
			Expression exp,
			Func<T, Expression> visitor )
		{
			return new ExpressionVisitor<T>( visitor ).Visit( exp );
		}

		public static Expression<TDelegate> Visit<TDelegate>(
			Expression<TDelegate> exp,
			Func<T, Expression> visitor )
		{
			return (Expression<TDelegate>)new ExpressionVisitor<T>( visitor ).Visit( exp );
		}

		public override Expression Visit( Expression exp )
		{
			if ( exp is T && visitor != null ) exp = visitor( (T)exp );

			return base.Visit( exp );
		}
	}
}